<?php

function rejoinurl($aurl)
{
  $url = $aurl['scheme'] . '://' . $aurl['host'];
  if (isset($aurl['port']))
    $url .= ':' . $aurl['port'];
  if (substr($aurl['path'], 0, 1) != '/')
    $aurl['path'] = '/' . $aurl['path'];
  $url .= $aurl['path'];
  if (isset($aurl['query']))
    $url .= '?' . $aurl['query'];
  return $url;
}

function showSummary($widget, $str, $priv = null)
{
  global $toolTip;

  if (strlen($str) > 102)
    $str = substr($str, 0, 100) . '...';

  $toolTip->set_tip($widget, $str, $priv);
}

function displayFeedData($url)
{
  global $feedTree, $recfeeds, $statusBar;

  $itemIndex = -1;
  $aurl = parse_url($url);

  if (empty($aurl['port']))
    $aurl['port'] = 80;
  if (empty($aurl['path']))
    $aurl['path'] = '/';

  $req = rejoinurl($aurl);

  if (getSetting('use-proxy'))
  {
    $server = getSetting('proxy-addr');
    $port = getSetting('proxy-port');
  }
  else
  {
    $port = $aurl['port'];
    $server = $aurl['host'];
  }

  $pfeed = fsockopen($server, $port, $errno, $errstr, getSetting('timeout-sec'));

  if (!$pfeed)
  {
    $statusBar->set_text('Connection Failed');
    alert("Could not initialize connection.\n\n\nError Code: $errno\n\nError Description: $errstr");
    return;
  }
  else
  {
    $displayedFeeds[$url] = true;
    $statusBar->set_text('Connection open; sending request...');
    fputs($pfeed, 'GET ' . $req . " HTTP/1.1\r\n");
    fputs($pfeed, 'Host: ' . $aurl['host'] . ':' . $aurl['port'] . "\r\n");
    fputs($pfeed, 'User-Agent: ' . getSetting('user-agent') . "\r\n");
    fputs($pfeed, 'Referer: ' . getSetting('referer') . "\r\n");
    fputs($pfeed, 'Accept: ' . getSetting('accept-types') . "\r\n");
    fputs($pfeed, "Accept-Encoding: \r\n");
    fputs($pfeed, "Connection: close\r\n");
    fputs($pfeed, "\r\n");
    while(!feof($pfeed))
    {
      $statusBar->set_text('Getting data...');
      $str = fgets($pfeed);
      if (preg_match('/^\w+/', $str))
      {
        $colonidx = strpos($str, ':');
        $fieldName = substr($str, 0, $colonidx ? $colonidx : strlen($str));
        $fieldVal = trim(substr($str, $colonidx + 1));
        if ($itemIndex != -1)
          $fieldName = strtolower($fieldName);
      }
      else if (preg_match('/^\r?\n$/', $str))
      {
        $itemIndex++;
        $fieldName = '';
      }
      else if (preg_match('/^\s+\w/', $str))
        $fieldVal .= "\n" . preg_replace('/\s+/', ' ', $str);

      $feeds[$itemIndex][$fieldName] = $fieldVal;
    }
    fclose($pfeed);
  }

  $nFeeds = count($feeds);
  $mainFeedIndex = 0;
  while (empty($feeds[$mainFeedIndex]['title']) && ($mainFeedIndex < $nFeeds))
    $mainFeedIndex++;

  list($key, $val) = each($feeds[-1]);
  preg_match('/\w+\/(\d\.\d)\s+(\d{3})\s+(.+)\r\n/', $key, $matches);
  list($match, $version, $code, $reason) = $matches;

  switch (substr($code, 0, 1))
  {
    case 2:
      $statusBar->set_text('Done (' . $reason . ')');
      break;
    case 3:
      getRemoteData($feeds[-1]['Content-Location'], $port_number);
      break;
    case 4:
    case 5:
      $statusBar->set_text('Done, with error: ' . $reason);
      break;
    default:
      $statusBar->set_text('Status ' . $code . ' isn\'t currently parsed');
  }

  $treeItem = &new GtkTreeItem($feeds[$mainFeedIndex]['title']);
  showSummary($treeItem, $feeds[$mainFeedIndex]['description']);
  $feedTree->append($treeItem);
  $subtree = &new GtkTree();
  $treeItem->connect('button-release-event', 'treeItemClicked', array($feeds[$mainFeedIndex], true));
  $treeItem->connect('key-press-event', 'treeItemPressed', array($feeds[$mainFeedIndex], true));
  $treeItem->connect('toggle', 'expandFeed');
  $treeItem->set_subtree($subtree);
  $treeItem->show();

  for ($idx = $mainFeedIndex + 1; $idx < $nFeeds - 1; $idx++)
  {
    if (isset($feeds[$idx]['title']))
    {
      $treeItem = &new GtkTreeItem($feeds[$idx]['title']);
      if (isset($feeds[$idx]['link']))
      {
        $curl = parse_url($feeds[$idx]['link']);
        $purl = parse_url($feeds[$mainFeedIndex]['link']);
        while (list($key, $val) = each($curl))
          $purl[$key] = $val;
        $feeds[$idx]['link'] = rejoinurl($purl);
      }
      if (isset($feeds[$idx]['description']))
        showSummary($treeItem, $feeds[$idx]['description']);
      $treeItem->connect('button-release-event', 'treeItemClicked', array($feeds[$idx], false));
      $treeItem->connect('key-press-event', 'treeItemPressed', array($feeds[$idx], false));
      $subtree->append($treeItem);
    }
    $treeItem->show();
  }
  if (empty($recfeeds[$feeds[$mainFeedIndex]['title']]))
    $recfeeds[$feeds[$mainFeedIndex]['title']] = rawurlencode($feeds[$mainFeedIndex]['title']) . ',' . rawurlencode($url) . ',' . 0;
}

function getRemoteFeed($url)
{
  global $statusBar;
  static $displayedFeeds;

  if (empty($displayedFeeds[$url]))
  {
    $statusBar->set_text('Opening connection...');
    displayFeedData($url);
  }
  else
    $statusBar->set_text('Done: This feed was already in your view');
}

?>