<?php

function createAppearancePage($notebook)
{
  $appearancePage = &new GtkVBox();
  $appearanceLabel = &new GtkLabel('Appearance');

  $rcCheckBox = &new GtkHBox();
  $appearancePage->pack_start($rcCheckBox);

  $rcCheckBtn = &new GtkCheckButton('Use a non-default resource file');
  $rcCheckBtn->set_active(getSetting('use-rc-file'));
  $rcCheckBox->pack_start($rcCheckBtn, false, false);

  $rcBox = &new GtkHBox();
  $appearancePage->pack_start($rcBox);

  $rcField = createEdit(getSetting('rc-file'));
  $rcField->set_sensitive(getSetting('use-rc-file'));
  $rcField->connect('changed', 'settingsEntryChanged', 'rc-file');
  $rcBox->pack_start($rcField);

  $rcBrowseBox = &new GtkHButtonBox();
  $rcBrowseBox->set_spacing(0);
  $rcBox->pack_start($rcBrowseBox, false, false, 5);

  $rcBrowse = &new GtkButton('Browse...');
  $rcBrowse->set_sensitive(getSetting('use-rc-file'));
  $rcBrowse->connect('clicked', 'createFileSelection', array($rcField, 'Browse for resource files...', 'rc-file', $rcField));
  $rcBrowseBox->pack_start($rcBrowse);

  $rcCheckBtn->connect('toggled', 'checkToggled', array(array($rcField, $rcBrowse), 'use-rc-file'));

  $notebook->append_page($appearancePage, $appearanceLabel);
}

function createHttpPage($notebook)
{
  $httpPage = &new GtkVBox();
  $httpLabel = &new GtkLabel('HTTP');
  $use_proxy = getSetting('use-proxy');

  $toBox = &new GtkHBox(false, 5);
  $httpPage->pack_start($toBox);

  $toPreLabel = &new GtkLabel('Timeout: ');
  $toBox->pack_start($toPreLabel, false);

  $toField = createEdit(getSetting('timeout-sec'));
  $toField->connect('changed', 'settingsEntryChanged', 'timeout-sec');
  $toBox->pack_start($toField);

  $toPostLabel = &new GtkLabel(' (seconds)');
  $toBox->pack_start($toPostLabel, false);

  $httpPageHSep1 = &new GtkHSeparator();
  $httpPage->pack_start($httpPageHSep1, false, false, 5);

  $browserBox = &new GtkHBox();
  $httpPage->pack_start($browserBox);

  $browserPreLabel = &new GtkLabel('HTTP Client: ');
  $browserBox->pack_start($browserPreLabel);

  $browserField = createEdit(getSetting('http-client'));
  $browserField->connect('changed', 'settingsEntryChanged', 'http-client');
  $browserBox->pack_start($browserField, false);

  $browserBrowse = &new GtkButton('Browse...');
  $browserBrowse->connect('clicked', 'createFileSelection', array($browserField, 'Browse for your preferred HTTP Client...', 'browser', $browserField));
  $browserBox->pack_start($browserBrowse);

  $proxyFrame = &new GtkFrame('Proxy');
  $httpPage->pack_start($proxyFrame);

  $proxyFrameBox = &new GtkVBox();
  $proxyFrame->add($proxyFrameBox);

  $useProxyCheck = &new GtkCheckButton('Use a proxy');
  $useProxyCheck->set_active($use_proxy);
  $proxyFrameBox->pack_start($useProxyCheck);

  $proxyBox = &new GtkHBox();
  $proxyBox->set_sensitive($use_proxy);
  $proxyFrameBox->pack_start($proxyBox);

  $useProxyCheck->connect('toggled', 'checkToggled', array(array($proxyBox), 'use-proxy'));

  $proxyLabels = &new GtkVBox();
  $proxyBox->pack_start($proxyLabels, false);

  $proxyAddrLabel = &new GtkLabel('Address: ');
  $proxyLabels->pack_start($proxyAddrLabel);

  $proxyPortLabel = &new GtkLabel('Port: ');
  $proxyLabels->pack_start($proxyPortLabel);

  $proxyFields = &new GtkVBox(false);
  $proxyBox->pack_start($proxyFields, false);

  $proxyAddrField = createEdit(getSetting('proxy-addr'));
  $proxyAddrField->connect('changed', 'settingsEntryChanged', 'proxy-addr');
  $proxyFields->pack_start($proxyAddrField);

  $proxyPortField = createEdit(getSetting('proxy-port'));
  $proxyPortField->connect('changed', 'settingsEntryChanged', 'proxy-port');
  $proxyFields->pack_start($proxyPortField);

  $notebook->append_page($httpPage, $httpLabel);
}

function createAdvHttpPage($notebook)
{
  $httpAdvPage = &new GtkVBox();
  $httpAdvPageLabel = &new GtkLabel('HTTP: Advanced');

  $httpAdvRespFrame = &new GtkFrame('Request Headers');
  $httpAdvPage->pack_start($httpAdvRespFrame);

  $httpAdvRespBox = &new GtkHBox(false);
  $httpAdvRespFrame->add($httpAdvRespBox);

  $labels = &new GtkVBox(false);
  $httpAdvRespBox->pack_start($labels, false);

  $uaLabel = &new GtkLabel('User Agent: ');
  $labels->pack_start($uaLabel);

  $acceptLabel = &new GtkLabel('Accept Types: ');
  $labels->pack_start($acceptLabel);

  $fields = &new GtkVBox(false);
  $httpAdvRespBox->pack_start($fields);

  $uaField = createEdit(getSetting('user-agent'));
  $uaField->connect('changed', 'settingsEntryChanged', 'user-agent');
  $fields->pack_start($uaField);

  $acceptField = createEdit(getSetting('accept-types'));
  $acceptField->connect('changed', 'settingsEntryChanged', 'accept-types');
  $fields->pack_start($acceptField);

  $notebook->append_page($httpAdvPage, $httpAdvPageLabel);
}

function createNotebook($parent)
{
  $notebook = &new GtkNotebook();
  $parent->pack_start($notebook);
  createAppearancePage($notebook);
  createHttpPage($notebook);
  createAdvHttpPage($notebook);
  return $notebook;
}

function showSettingsDialog()
{
  global $settings;
  $old_settings = $settings;

  $dialog = &new GtkWindow(GTK_WINDOW_DIALOG);
  $dialog->set_title('Settings...');
  $dialog->set_policy(false, false, true);
  $dialog->connect('delete-event', 'settingsDlgDel', array(true, $old_settings));

  $dlgBox = &new GtkVBox();
  $dialog->add($dlgBox);

  $notebook = createNotebook($dlgBox);

  $buttonBox = &new GtkHButtonBox();
  $buttonBox->set_layout(GTK_BUTTONBOX_SPREAD);
  $buttonBox->set_spacing(3);
  $dlgBox->pack_start($buttonBox, false, false, 2);

  $okBtn = &new GtkButton('OK');
  $okBtn->set_flags(GTK_CAN_DEFAULT);
  $okBtn->connect('clicked', 'killWidget', $dialog);

  $cancelBtn = &new GtkButton('Cancel');
  $cancelBtn->connect('clicked', 'cancelSettingsChange', $old_settings);
  $cancelBtn->connect('clicked', 'killWidget', $dialog);

  $buttonBox->pack_start($okBtn);
  $buttonBox->pack_start($cancelBtn);

  $okBtn->grab_default();

  $dialog->show_all();
}

?>